# frozen_string_literal: true

require "capybara"

# Extends Capybara's Actions to create end user documentation
module UserDocs
  extend Capybara::Node::Actions

  def before_setup
    super

    check_installation

    @class_name = class_name.chomp("Procedure")
    @method = name.sub("test_", "")
    make_version_data_directory
    make_version_index
    Capybara.save_path =
      Rails.root.join(".docs/assets/#{version}/#{klass_name}/#{@method}")

    @json = { name: @method.humanize, steps: [] }
  end

  def before_teardown
    path = ".docs/_data/versions/#{version}/#{klass_name}/#{@method}.json"
    File.write(Rails.root.join(path), @json.to_json)

    super
  end

  private

  def check_installation
    return if Dir.exist?(Rails.root.join("docs"))

    # :nocov:
    abort("ERROR: Run 'rails g user_docs:install.'")
    # :nocov:
  end

  def version
    Rails.application.class.module_parent::VERSION.tr(".", "_")
  end

  def klass_name
    @class_name.underscore
  end

  def make_version_data_directory
    data_path = Rails.root.join(".docs/_data/versions/#{version}/#{klass_name}")
    FileUtils.mkdir_p(data_path)
  end

  # rubocop:disable Metrics/MethodLength
  def make_version_index
    directory = Rails.root.join(".docs/versions/#{version}")
    output_file = directory + "index.html"
    entry_file = Rails.root.join(".docs/index.html")

    FileUtils.mkdir_p(directory)
    File.write(
      output_file,
      <<~HTML
        ---
        layout: default
        version: #{version.tr!('_', '.')}
        ---
        {% include index.html %}
      HTML
    )

    FileUtils.cp_r(output_file, entry_file, remove_destination: true)
  end
  # rubocop:enable Metrics/MethodLength

  def click_on(link)
    super
    @json[:steps] << "Click on **#{link}**."
  end

  def fill_in(text, with:)
    super
    @json[:steps] << "Fill in **#{text}** with *#{with}*."
  end

  def choose(label)
    super
    @json[:steps] << "Select radio button with *#{label}*."
  end

  def accept_alert
    page.accept_alert
    @json[:steps] << "Click **OK**."
  end

  def save_screenshot(image = Time.now.to_i.to_s)
    image += ".png"
    super

    path = "/docs/assets/#{version}/#{klass_name}/#{@method}/#{image}"
    @json[:steps] << "<img src='#{path}' class='img-fluid'>"
  end

  def comment(string)
    @json[:steps] << string
  end
end
